//
//  STTimelineInstance.h
//  Kahuna
//
//  Created by Davyd Geyl on 6/08/09.
//  Copyright 2009 Sportstec Ltd. All rights reserved.
//

#import <Cocoa/Cocoa.h>

@import SCNFoundation.STFramedObject;
@import SCNFoundation.STProtocols;



@class STTimelineInstanceTag;
@class STTimelineEvent;

extern NSTimeInterval const STMinimalInstanceInterval;
extern NSString* const STTimelineInstanceUpdatedLabelsNotification;
extern NSString* const STTimelineInstanceEventLabelKey;
extern NSString* const STTimelineInstanceLabelsKey;
extern NSString* const STTimelineInstanceTagGroupsKey;
extern NSString* const STTimelineInstanceIDKey;

@protocol STTimelineInstanceCurrentEndTimeDelegate

- (NSTimeInterval) timelineCurrentTime;
- (NSTimeInterval) timelineContentDuration;

@end

@interface STTimelineInstance : STFramedObject <STTimelineInstanceInterface, NSCoding, STHeldObjectInterface>

+ (instancetype)timelineInstanceWithID:(NSInteger )instanceID startTime:(NSTimeInterval)startTime duration:(NSTimeInterval)duration;
+ (NSArray*)validInstancesWithInstances:(NSArray*)instances contentDuration:(NSTimeInterval)duration;

@property (nonatomic, assign) NSInteger instanceID;
// TODO: should deprecate this instanceID as it is only appliciable in local tagging
// Use identifier property instead

@property (nonatomic, assign) NSTimeInterval startTime;
@property (nonatomic, assign) NSTimeInterval duration;
@property (nonatomic, assign) NSTimeInterval endTime;
@property (nonatomic, assign) STTimelineEvent* timelineEvent;
@property (nonatomic, readonly) NSString* code;
@property (nonatomic, weak) id<STTimelineInstanceCurrentEndTimeDelegate> currentEndTimeDelegate;

@property (nonatomic, readonly, copy) NSArray* timelineInstanceTags;
@property (nonatomic, strong) NSString* notes;

@property (nonatomic, readonly) NSString *identifier;
@property (nonatomic, readonly) NSString *JSONStringRepresentation;
@property (nonatomic, readonly, copy) NSString *tagsString;

- (instancetype)initWithID:(NSInteger)instanceID startTime:(NSTimeInterval)startTime duration:(NSTimeInterval)duration;
- (instancetype)initWithID:(NSInteger)instanceID startTime:(NSTimeInterval)startTime duration:(NSTimeInterval)duration instanceTags:(NSArray *)instanceTags;
- (instancetype)initWithJSONRepresentation:(NSString *)JSONRepresentation;

- (void)setTimelineInstanceTagsWithTimelineInstance:(STTimelineInstance*)timelineInstance;

- (BOOL)isEqualToInstance:(STTimelineInstance*)instance;

- (void)setUserStartTime:(NSTimeInterval)time previousStartTime:(NSTimeInterval)previousStratTime duration:(NSTimeInterval)duration previousDuration:(NSTimeInterval)previousDuration;

- (void)setUserEndTime:(NSTimeInterval)time;
- (void)setUserEndTime:(NSTimeInterval)time previousEndTime:(NSTimeInterval)previousEndTime;
- (void)setEndTimeAndNotify:(NSTimeInterval)time;
- (void)setUserDuration:(NSTimeInterval)duration previousDuration:(NSTimeInterval)previousDuration;
- (NSTimeInterval)validatedDuration:(NSTimeInterval)duration;

- (BOOL)containsTime:(NSTimeInterval)time;
- (BOOL)isOverlappingInstance:(STTimelineInstance*)instance;

- (STTimelineInstance*)validInstanceWithContentDuration:(NSTimeInterval)duration;

#pragma mark - Update Tags
/**
 Adds a tag to the instance. Does NOT notify UI.

 @param tag tag to add
 */
- (void)addTag:(STTimelineInstanceTag*)tag;

/**
 Adds an array of tags to the instance. Does NOT notify UI.

 @param tags an array of tags to add
 */
- (void)addTags:(NSArray*)tags;

/**
 Removes a tag from the instance. Does NOT notify UI.

 @param tags tag to remove
 */
- (void)removeTag:(STTimelineInstanceTag*)tags;

/**
 Removes an array of tags from the instance. Does NOT notify UI.

 @param tags an array of tags to remove
 */
- (void)removeTags:(NSArray*)tags;

#pragma mark - Update Tags and Notify UI
/**
 Adds a tag to the instance and notifies UI

 @param tag tag to add
 */
- (void)addTagAndNotify:(STTimelineInstanceTag *)tag;

/**
 Adds an array of tags to the instance and notifies UI

 @param tags an array of tags to add
 */
- (void)addTagsAndNotify:(NSArray *)tags;

/**
 Remove a tag from the instance and notify UI

 @param tag tag to remove
 */
- (void)removeTagAndNotify:(STTimelineInstanceTag *)tag;

/**
 Removes an array of tags from the instance and notifies UI

 @param tags an array of tags to remove
 */
- (void)removeTagsAndNotify:(NSArray *)tags;

#pragma mark - Update Tags (Undoable)
/**
 Adds a tag to the instance, notifies the UI, and pushes an undo action to the instance's undo stack.

 @param tag tag to add
 */
- (void)addUserTag:(STTimelineInstanceTag*)tag;

/**
 Adds an array of tags to the instance, notifies the UI, and pushes an undo action to the instance's undo stack.

 @param tags array of tags to add
 */
- (void)addUserTags:(NSArray*)tags;

/**
 Removes a tag from the instance, notifies the UI, and pushes an undo action to the instance's undo stack

 @param tag tag to remove
 */
- (void)removeUserTag:(STTimelineInstanceTag*)tag;

/**
 Removes an array of tags from the instance, notifies the UI, and pushes an undo action to the instance's undo stack

 @param tags array of tags to remove
 */
- (void)removeUserTags:(NSArray*)tags;

#pragma mark -

- (NSUInteger)countOfTags;

- (BOOL)isOpen; 
- (void)setReopen;

- (NSUInteger)instanceIndex;
- (NSString *)displayLabel;

- (NSString*)formattedStartTimeShort;
- (NSString*)formattedEndTimeShort;

@end


